require 'spec_helper'

RSpec.describe GraphQuery do
  describe "transform_to_array" do
    it "support plain fields" do
      expect(GraphQuery.new("albums,posts").transform_to_array).to eq [:albums, :posts]
      expect(GraphQuery.new("name,picture,tags").transform_to_array).to eq [:name, :picture, :tags]
    end

    it "support nested fields" do
      expect(GraphQuery.new("albums{name}").transform_to_array).to eq [{
        albums:[:name]
      }]
    end

    it "support plain + nested fields" do
      expect(GraphQuery.new("albums{name,photos},posts").transform_to_array).to eq [{
        albums:[
          :name,
          :photos
        ]},
        :posts
      ]

      expect(GraphQuery.new("posts,albums{name,photos}").transform_to_array).to eq [
        :posts,
        {
          albums:[
            :name,
            :photos
          ]
        }
      ]
    end

    it "support multiple neseted fields" do
      expect(GraphQuery.new("albums{name,photos{name,picture,tags}},posts").transform_to_array).to eq [{
        albums:[
          :name,
          photos:[
            :name,
            :picture,
            :tags
          ]
        ]},
        :posts
      ]

      expect(GraphQuery.new("albums{name,photos{name,picture,tags}},posts{author}").transform_to_array).to eq [{
        albums:[
          :name,
          photos:[
            :name,
            :picture,
            :tags
          ]
        ]},{
          posts: [:author]
        }
      ]
    end

    it "raise error if open count <> close count" do
      expect{
        GraphQuery.new("albums{name").transform_to_array
      }.to raise_error(GraphQuery::OpenBraceError)

      expect{
        GraphQuery.new("albums}name").transform_to_array
      }.to raise_error(GraphQuery::CloseBraceError)
    end
  end
end